"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("superagent");
const deep_object_diff_1 = require("deep-object-diff");
const THIRTY_DAYS = 1000 * 60 * 60 * 24 * 30;
const defaultOptions = {
    url: 'http://localhost:3100',
    domain: 'sr.dev',
};
function Session(optionsIn) {
    const options = Object.assign({}, defaultOptions, optionsIn);
    const doPrintDebug = /session-middleware/.test(process.env.DEBUG || '');
    // tslint:disable-next-line:no-console
    const print = (str) => doPrintDebug && console.log(str);
    // Ping auth service
    request
        .get(`${options.url}`)
        .on('error', () => {
        throw new Error("Couldn't reach auth service.");
    })
        .then(response => {
        if (response.text !== 'What are you doing here?') {
            throw new Error(`Auth service responded incorrectly (${response.text}).`);
        }
        print('Auth service responded correctly');
    });
    async function getSession(sid) {
        let response;
        try {
            response = await request.get(`${options.url}/api/session/${sid}`);
        }
        catch (err) {
            response = { body: null, status: err.status || 500 };
        }
        if (response.status === 404) {
            return createSession();
        }
        return response.body;
    }
    async function createSession() {
        let response;
        try {
            response = await request.post(`${options.url}/api/session`);
        }
        catch (err) {
            response = { body: null };
        }
        return response.body;
    }
    async function updateSession(sid, obj) {
        let response;
        try {
            response = await request
                .patch(`${options.url}/api/session/${sid}`)
                .send(obj);
        }
        catch (err) {
            response = { body: null };
        }
        return response.body;
    }
    return async function Middleware(ctx, next) {
        const sid = ctx.cookies.get('sid');
        let session;
        if (sid) {
            session = await getSession(sid);
        }
        else {
            session = await createSession();
        }
        ctx.session = Object.assign({ id: session.id, maxAge: session.maxAge, user: session.user }, session.session);
        ctx.cookies.set('sid', session.id, {
            maxAge: THIRTY_DAYS,
            domain: '.' + options.domain,
            secure: process.env.NODE_ENV === 'production',
        });
        await next();
        const _a = ctx.session, { id, maxAge, user } = _a, rest = __rest(_a, ["id", "maxAge", "user"]);
        const ctxSession = { id, maxAge, user, session: rest };
        const difference = deep_object_diff_1.diff(session, ctxSession);
        if (Object.keys(difference).length > 0) {
            await updateSession(id, deep_object_diff_1.diff(session, ctxSession));
        }
    };
}
exports.Session = Session;
//# sourceMappingURL=index.js.map